<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

add_action( 'admin_init', array( 'Scarcity_Samurai_Auto_Responder_Integrator', 'add_ajax' ) );

class Scarcity_Samurai_Auto_Responder_Integrator {

	private static $auto_responders = array(
		'aweber' => array(
			// Auto responder's name that is used in output messages. Should be
			// exactly the same like it appears on the auto responder's web site.
			'name' => 'AWeber',
			// Auto responder's web site URL. Can be String, Array, or null.
			// See examples of each below.
			'site_url' => '//aweber.com',
			// Regular expression that is used to identify the auto responder.
			'action_regex' => '/aweber\.com/',
			// When 'token_field_wrapper_level' is null, 'token_field_name' is the
			// 'name' of the custom token field we inject into the form.
			// Otherwise, 'token_field_name' is a jQuery-like selector of the custom
			// token field. See: http://simplehtmldom.sourceforge.net/manual.htm#section_find
			'token_field_name' => 'custom token',
			// When exporting the web form in the auto responder's web site, some
			// auto responders include the custom fields, while others don't.
			// If the custom token field is part of the exported form,
			// 'token_field_wrapper_level' should be >= 0. This number is used to
			// hide the custom token field and its label. If there is no label, and
			// the only thing we need to hide is the custom token field itself,
			// 'token_field_wrapper_level' should be 0. If the parent element of the
			// custom token field contains the label, and we want to hide it,
			// 'token_field_wrapper_level' should be 1.
			// If the custom token field is NOT part of the exported form,
			// 'token_field_wrapper_level' should be null.
			'token_field_wrapper_level' => null,
			// This is the placeholder that the auto responder uses for the custom
			// 'token' field. If we put this string in an email that auto responder
			// sends, the auto responder should replace this placeholder with the
			// value of the custom 'token' field.
			// It can be either a string or a function that returns a string.
			'token_placeholder' => '{!custom token}',
			// Link to Knowledgebase article that explains the necessary configuration
			// that needs to be done in the auto responder's web site to use Scarcity
			// Samurai.
			'configuration_instructions_url' => '//support.noblesamurai.com/entries/21509854-Setting-up-Aweber-to-work-with-Scarcity-Samurai'
		),
		'emailsamurai' => array(
			'name' => 'Email Samurai',
			'site_url' => null,
			'action_regex' => '/lists\.noblesamurai\.com/',
			'token_field_name' => 'custom token',
			'token_field_wrapper_level' => null,
			'token_placeholder' => '{!custom token}',
			'configuration_instructions_url' => null
		),
		'getresponse' => array(
			'name' => 'GetResponse',
			'site_url' => '//getresponse.com',
			'action_regex' => '/\.getresponse\.com/',
			'token_field_name' => '[name=custom_token]',
			'token_field_wrapper_level' => 2,
			'token_placeholder' => '[[cus token]]',
			'configuration_instructions_url' => '//support.noblesamurai.com/entries/21509359-Setting-up-GetResponse-to-work-with-Scarcity-Samurai'
		),
		'infusionsoft' => array(
			'name' => 'Infusionsoft',
			'site_url' => '//infusionsoft.com',
			'action_regex' => '/infusionsoft\.com/',
			'token_field_name' => '[name^=inf_custom_token]',
			'token_field_wrapper_level' => 0,
			'token_placeholder' => array( __CLASS__, 'get_infusionsoft_token_placeholder'),
			'configuration_instructions_url' => '//support.noblesamurai.com/entries/21509864-Setting-up-InfusionSoft-to-work-with-Scarcity-Samurai'
		),
		'mailchimp' => array(
			'name' => 'MailChimp',
			'site_url' => '//mailchimp.com',
			'action_regex' => '/\.list-manage.*\.com/',
			'token_field_name' => 'TOKEN',
			'token_field_wrapper_level' => null,
			'token_placeholder' => '*|TOKEN|*',
			'configuration_instructions_url' => '//support.noblesamurai.com/entries/21512065-Setting-up-MailChimp-to-work-with-Scarcity-Samurai'
		),
		'sendpepper' => array(
			'name' => 'SendPepper/OfficeAutoPilot',
			'site_url' => array(
				'SendPepper' => '//sendpepper.com',
				'OfficeAutoPilot' => '//officeautopilot.com'
			),
			'action_regex' => '/moon-ray\.com/',
			'token_field_name' => '[name^=token]',
			'token_field_wrapper_level' => 1,
			'token_placeholder' => '[token]',
			'configuration_instructions_url' => '//support.noblesamurai.com/entries/21680760-Setting-up-SendPepper-or-Office-Auto-Pilot-to-work-with-Scarcity-Samurai'
		),
	);

	private static function get_auto_responder_by_form_action( $form_action ) {
		foreach ( self::$auto_responders as $auto_responder => $auto_responder_settings )  {
			if ( preg_match( $auto_responder_settings['action_regex'], $form_action ) ) {
				return $auto_responder;
			}
		}

		return null;
	}

	public static function supported_auto_responders() {
		return array_keys( self::$auto_responders );
	}

	public static function supported_auto_responders_message() {
		$supported_auto_responders = array();

		foreach ( self::$auto_responders as $auto_responder ) {
			$site_url = $auto_responder['site_url'];

			if ( is_string( $site_url ) ) {
				$supported_auto_responders[] = array(
					'name' => $auto_responder['name'],
					'url' => $site_url
				);
			} else if ( is_array( $site_url ) ) {
				foreach ( $site_url as $name => $url ) {
					$supported_auto_responders[] = array(
						'name' => $name,
						'url' => $url
					);
				}
			}
		}

		function sort_by_name( $first, $second ) {
			return strcasecmp( $first['name'], $second['name']);
		}

		usort($supported_auto_responders, 'sort_by_name');

		$supported_auto_responders_count = count( $supported_auto_responders );

		$message = 'We currently only support ';

		foreach ( $supported_auto_responders as $index => $supported_auto_responder ) {
			$name = $supported_auto_responder['name'];
			$url = $supported_auto_responder['url'];

			$message .= "<a href='$url' target='_blank'>$name</a>";

			if ( $index === $supported_auto_responders_count - 1 ) {
				$message .= '.';
			} else if ( $index === $supported_auto_responders_count - 2 ) {
				$message .= ', and ';
			} else {
				$message .= ', ';
			}
		}

		echo $message;
	}

	public static function get_auto_responder_short_name( $auto_responder ) {
		if ( ( $auto_responder === null ) ||
		     ( ! isset( self::$auto_responders[$auto_responder] ) ) ) {
			return '';
		}

		return self::$auto_responders[$auto_responder]['name'];
	}

	public static function get_auto_responder_token_placeholder( $auto_responder, $page_id = null ) {
		if ( ( $auto_responder === null ) ||
		     ( ! isset( self::$auto_responders[$auto_responder] ) ) ) {
			return null;
		}

		$token_placeholder = self::$auto_responders[$auto_responder]['token_placeholder'];

		if ( is_string( $token_placeholder ) ) {
			return $token_placeholder;
		}

		if ( is_array( $token_placeholder ) &&
		     method_exists( $token_placeholder[0], $token_placeholder[1] ) ) {
			return call_user_func( $token_placeholder, $page_id );
		}

		return null;
	}

	private static function get_infusionsoft_token_placeholder( $page_id ) {
		$default_token_placeholder = '~Contact._token~';

		// We need to find the 'id' of the token field here
		$page = Scarcity_Samurai_Model::get( 'Page' )->find( $page_id );
		$campaign_id = $page['campaign_id'];
		$squeeze_page = Scarcity_Samurai_Model::get( 'Campaign' )->squeeze_page( $campaign_id );
		$forms = self::get_page_forms( $squeeze_page['id'] );

		if ( empty( $forms ) ) {
			return $default_token_placeholder;
		}

		$form = $forms[0];

		$token_field_name = self::$auto_responders['infusionsoft']['token_field_name'];
		$token_field = $form->find( $token_field_name, 0 );

		if ( $token_field->id === false ) {
			return $default_token_placeholder;
		}

		return "~Contact.{$token_field->id}~";
	}

	public static function get_auto_responder_configuration_instructions_link( $auto_responder ) {
		if ( ( $auto_responder === null ) ||
		     ( ! isset( self::$auto_responders[$auto_responder] ) ) ) {
			return '';
		}

		$url = self::$auto_responders[$auto_responder]['configuration_instructions_url'];
		$short_name = self::$auto_responders[$auto_responder]['name'];
		$result = "$short_name Configuration Instructions";

		if ( $url !== null ) {
			$result = "<a href='$url' target='_blank'>$result</a>";
		}

		return $result;
	}

	private static function hide_element( $element ) {
		// The element is going to be hidden, so we can override its style.
		$element->style = 'display: none';
	}

	// When $hidden_fields is provided:
	//   - $html is the HTML of the Optimize Press form
	//   - The return value is an updated version of $hidden_fields
	// When $hidden_fields is not provided:
	//   - $html is the HTML of the whole page
	//   - The return value is an updated version of $html
	public static function update_html( $html, $hidden_fields = null ) {
		$optimize_press = ( $hidden_fields !== null );

		// If the current page is not part of a campaign, exit.
		$current_page = Scarcity_Samurai_Helper::current_page();
		$campaign_id = $current_page['campaign_id'];

		if ( $campaign_id === null ) {
			return $optimize_press ? $hidden_fields : $html;
		}

		$page_id = $current_page['id'];

		$html = simple_html_1_11_str_get_html( $html );
		$forms = $html->find( 'form' );

		// If no forms were found, exit.
		if ( empty( $forms ) ) {
			return $optimize_press ? $hidden_fields : $html;
		}

		if ( $optimize_press ) {
			$hidden_fields = simple_html_1_11_str_get_html($hidden_fields);
		}

		foreach ( $forms as $form ) {
			$auto_responder = self::get_auto_responder_by_form_action( $form->action );

			// If the auto responder is unknown, leave the form as is.
			if ( $auto_responder === null ) {
				if ( $optimize_press ) {
					return $hidden_fields;
				}

				continue;
			}

			$auto_responder_settings = self::$auto_responders[$auto_responder];

			$nonce_field = "<input type='hidden' name='ss_nonce' value='" . wp_create_nonce( 'ss-subscribe-user' ) . "'>";
			$page_id_field = "<input type='hidden' name='ss_page_id' value='$page_id'>";

			// We generate a Global Unique IDentifier that will be stored in auto
			// responder's database along with the user's email. This GUID will be
			// stored in SS database as well.
			$guid = Scarcity_Samurai_Security::generate_guid();

			if ( $auto_responder === 'aweber' ) {
				// Remove <input type="hidden" name="meta_forward_vars" ... />, if found
				$meta_forward_vars = $optimize_press ?
				                     $hidden_fields->find( '[name=meta_forward_vars]', 0 ) :
				                     $form->find( '[name=meta_forward_vars]', 0 );

				if ( $meta_forward_vars !== null ) {
					$meta_forward_vars->outertext = ''; // Remove it!
				}
			}

			$token_field_name = $auto_responder_settings['token_field_name'];
			$token_field_wrapper_level = $auto_responder_settings['token_field_wrapper_level'];

			if ( $token_field_wrapper_level === null ) {
				$token_field = "<input type='hidden' name='$token_field_name' value='$guid' />";
			} else {
				// Find the custom token field
				$token_field = $form->find( $token_field_name, 0 );

				$empty_token_field = true;

				if ( $optimize_press ) {
					if ( $token_field === null ) {
						$token_field = $hidden_fields->find( $token_field_name, 0 );
					} else {
						$empty_token_field = false;
					}
				}

				if ( $token_field === null ) {
					$url = $auto_responder_settings['configuration_instructions_url'];

					wp_die( 'Custom <strong>token</strong> field is missing.<br>' .
					        'Please configure ' . $auto_responder_settings['name'] .
					        ' as described here: ' .
					        self::get_auto_responder_configuration_instructions_link( $auto_responder ) );
				}

				// Set the token field value
				$token_field->value = $guid;

				if ( $optimize_press ) {
					// Hide the token input field.
					self::hide_element( $token_field );
				} else {
					// Hide the custom token field and its label.
					$element_to_hide = $token_field;

					for ( $i = 0; $i < $token_field_wrapper_level; $i++ ) {
						$element_to_hide = $element_to_hide->parent();
					}

					self::hide_element( $element_to_hide );
				}

				if ( $empty_token_field ) {
					// Custom token field exists. Don't add another one.
					$token_field = '';
				}
			}

			if ( $optimize_press ) {
				return $hidden_fields->outertext .
					$nonce_field .
					$page_id_field .
					$token_field;
			} else {
				$form->innertext =
					$nonce_field .
					$page_id_field .
					$token_field .
					$form->innertext;
			}
		}

		return $html;
	}

	private static function get_page_forms( $page_id ) {
		if ( ! Scarcity_Samurai_Page::contains_opt_in_form( $page_id ) ) {
			return array();
		}

		$page_forms = array();

		// Check non Optimize Press forms in page content
		$content = Scarcity_Samurai_Helper::get_page_content_by_id( $page_id );

		if ( ! empty( $content ) ) {
			$content = simple_html_1_11_str_get_html( $content );
			$page_forms = $content->find( 'form' );
		}

		// Check Optimize Press form in page meta data
		$current_theme = wp_get_theme();

		if ( $current_theme->Name === 'OptimizePress' ) {
			$optimize_press_form = get_post_meta( $page_id, '_optthemes_webformcodehtml' );

			if ( ! empty( $optimize_press_form ) ) {
				$optimize_press_form = simple_html_1_11_str_get_html( $optimize_press_form[0] );
				$optimize_press_form = $optimize_press_form->find( 'form', 0 );

				if ( $optimize_press_form !== null ) {
					array_push( $page_forms, $optimize_press_form );
				}
			}
		}

		return $page_forms;
	}

	public static function get_page_forms_auto_responders( $page_id ) {
		$auto_responders = array();
		$forms = self::get_page_forms( $page_id );

		foreach ( $forms as $form ) {
			$auto_responder = self::get_auto_responder_by_form_action( $form->action );

			if ( $auto_responder !== null ) {
				$auto_responders[] = $auto_responder;
			}
		}

		return $auto_responders;
	}

	public static function add_ajax() {
		add_action( 'wp_ajax_ss_subscribe_user', array( __CLASS__, 'subscribe_user' ) ); // For logged in users
		add_action( 'wp_ajax_nopriv_ss_subscribe_user', array( __CLASS__, 'subscribe_user' ) ); // For not logged in users
	}

	public static function subscribe_user() {
		$args = wp_parse_args( $_REQUEST, array(
			'nonce' => '',
			'page_id' => '',
			'email' => '',
			'token' => ''
		) );

		extract( $args );

		if ( ! wp_verify_nonce( $nonce, 'ss-subscribe-user' ) ) {
			wp_send_json_error( "You don't have permission to do that." );
		}

		if ( ! Scarcity_Samurai_User::subscribe( $email, $token, $page_id ) ) {
			wp_send_json_error( "Couldn't subscribe user." );
		}

		Scarcity_Samurai_Security::set_cookie( 'scarcity_samurai_token', $token );

		wp_send_json_success();
	}

}
