<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

require_once('../../../../wp-load.php');

if (!check_ajax_referer('counters', 'security_token', false)) {
	exit;
}

function error($error_message, $field_name = null) {
	echo json_encode(array(
		'status' => 'error',
		'errors' => array(array(
			'field_name' => $field_name,
			'error_message' => $error_message
		))
	));
	exit;
}

function validate_url($url) {
	return (preg_match('/\w+\.\w{2,3}|localhost/', $url) === 1);
}

$supported_directions = array('up', 'down');

$counter_id = Scarcity_Samurai_Helper::get_request('counter_id');
$direction = Scarcity_Samurai_Helper::get_request('direction');
$max = Scarcity_Samurai_Helper::get_request('max');
$thank_you_page = Scarcity_Samurai_Helper::get_request('thank_you_page');
$expired_page = Scarcity_Samurai_Helper::get_request('expired_page');

// Validate counter_id
if ($counter_id !== '') {
	$counter = Scarcity_Samurai_Model::get( 'Counter' ) ->find( $counter_id );

	if ($counter === null) {
		exit;
	}
}

// Validate direction
if (!in_array($direction, $supported_directions)) {
	error('Please select the counter type.');
}

// Validate max
if (($direction === 'down') && ($max === '')) {
	error('Cannot be blank', "down_counter_max");
}

if (($max !== '') && !ctype_digit($max)) {
	error('Must be a number', "${direction}_counter_max");
}

if ($max !== '') {
	$max = intval($max);

	if ($max === 0) {
		error('Cannot be zero', "${direction}_counter_max");
	}
}

// Validate thank_you_page
if ($thank_you_page === '') {
	error('Cannot be blank', "${direction}_counter_thank_you_page");
} else {
	if (!validate_url($thank_you_page)) {
		error('Must be a valid URL', "${direction}_counter_thank_you_page");
	}

	if (!preg_match('/^https?:/', $thank_you_page)) {
		$thank_you_page = "http://$thank_you_page";
	}
}

// Validate expired_page
if ($expired_page !== '') {
	if (!validate_url($expired_page)) {
		error('Must be a valid URL', "${direction}_counter_expired_page");
	}

	if (($direction === 'up') && ($max === '') && ($expired_page !== '')) {
		error('Must be empty when counter has no limit', "up_counter_expired_page");
	}

	if (!preg_match('/^https?:/', $expired_page)) {
		$expired_page = "http://$expired_page";
	}
}

// Prepare the data
$data = array(
	'direction' => $direction,
	'min' => 0
);

if ($max !== '') {
	$data['max'] = $max;
}

if ($thank_you_page !== '') {
	$data['thank_you_page'] = $thank_you_page;
}

if ($expired_page !== '') {
	$data['expired_page'] = $expired_page;
}

// Create or edit the counter
if ($counter_id === '') {
	$counter_id = Scarcity_Samurai_Model::get( 'Counter' )->insert( array(
		'id' => Scarcity_Samurai_Security::generate_guid(),
		'value' => 0,
		'data' => $data
	));

	if ($counter_id === false) {
		error('Unexpected error occurred. Adding new counter failed.');
	}
} else {
	if ( ! Scarcity_Samurai_Model::get( 'Counter' )->update( array( 'data' => $data ), $wpdb->prepare( 'id = %s', $counter_id ) ) ) {
		error('Unexpected error occurred. Updating a counter failed.');
	}
}

// Send the response
$counter = Scarcity_Samurai_Model::get( 'Counter' )->find( $counter_id );

ob_start();
include('../html/templates/counter.php');
$counter_row_html = ob_get_clean();

echo json_encode(array(
	'status' => 'ok',
	'counter_row_html' => $counter_row_html
));
