<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

function error() {
	echo '[Error]';
	exit;
}

// Legal combinations:
//   - token + page_id   (opt_in)
//   - page_id           (fixed)
//   - deadline
//   - time
function check_token_and_params($token, $params, $type) {
	if (array_key_exists('deadline', $params) ||
	    array_key_exists('time', $params)) {
		return;
	}

	if (!array_key_exists('page_id', $params)) {
		error();
	}

	if ((($type == 'opt_in') && ($token !== '')) ||
	    ($type == 'fixed')) {
		return;
	}

	error();
}

function set_timer_value($token, &$params, $type) {
	if (array_key_exists('time', $params)) {
		$params['timer_value'] = $params['time'];
	} else if (array_key_exists('deadline', $params)) {
		$params['timer_value'] = $params['deadline'] - time();
	} else if (array_key_exists('page_id', $params)) {
		if ($type == 'opt_in') {
			$user_id = Scarcity_Samurai_Helper::get_user_id($token);

			if ($user_id === null) {
				error();
			}
		}

		$page = Scarcity_Samurai_Model::get( 'Page' )->find( $params['page_id'] );

		if ($page === null) {
			error();
		}

		$available_until = $page['available_until'];

		if ($available_until === null) {
			error();
		}

		switch ($type) {
			case 'opt_in':
				$record = Scarcity_Samurai_Model( 'Users_Subscriptions' )->find_by(array(
					'user_id' => $user_id,
					'page_id' => $page['id']
				));

				if ($record === null) {
					error();
				}

				$subscription_time = $record['subscription_time'];

				$params['timer_value'] = $subscription_time + $available_until['value'] - time();
				break;

			case 'fixed':
				$params['timer_value'] = $available_until['value'] - time();
				break;
		}
	}
}

require_once('../../../../wp-load.php');

$cache_time = 5; // Cache GIF for 5 seconds
$cache_filename = sys_get_temp_dir() . '/scarcity_samurai_timer_cache.gif';

$params = Scarcity_Samurai_Helper::get_request('params');
$params = Scarcity_Samurai_GIF_Generator_Helper::decode_params($params);

if ($params === null) {
	error();
}

$token = Scarcity_Samurai_Helper::get_request('token');

if (array_key_exists('page_id', $params)) {
	$page = Scarcity_Samurai_Model( 'Page' )->find($params['page_id']);
	$type = $page['available_until']['type'];
} else {
	$type = null;
}

check_token_and_params($token, $params, $type);

header('Content-type: image/gif');
header("Expires: Mon, 26 Jul 1990 05:00:00 GMT");
header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Cache-Control: post-check=0, pre-check=0", false);
header("Pragma: no-cache");

if (($params['cache'] === true) && file_exists($cache_filename) &&
    (time() - filemtime($cache_filename) < $cache_time)) {
	readfile($cache_filename);
	exit;
}

set_timer_value($token, $params, $type);

require_once('../gif-generator/timer-gif-generator.php');

$timer_gif_generator = new Scarcity_Samurai_Timer_GIF_Generator($params);
$timer_gif = $timer_gif_generator->get_gif();

// Cache the GIF
file_put_contents($cache_filename, $timer_gif, LOCK_EX);

// Output the GIF
echo $timer_gif;
