<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

require_once('gif-generator.php');

class Scarcity_Samurai_Counter_GIF_Generator extends Scarcity_Samurai_GIF_Generator {

	function __construct($params) {
		parent::__construct($params);

		$this->counter_id = $params['counter_id'];

		$counter = Scarcity_Samurai_Model( 'Counter' )->find( $this->counter_id );
		$counter_direction = $counter['data']['direction'];
		$this->counter_expired = Scarcity_Samurai_Counter::expired($counter);
		$this->counter_text = Scarcity_Samurai_Counter::display_value($counter);

		// Text parameters
		$this->text_font = Scarcity_Samurai_GIF_Generator_Helper::get_font_param('text_font', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['text_font']);
		$this->text_font_size = Scarcity_Samurai_GIF_Generator_Helper::get_param('text_font_size', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['text_font_size']);
		$this->text_color = Scarcity_Samurai_GIF_Generator_Helper::get_param('text_color', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['text_color']);

		// Text before counter
		switch ($counter_direction) {
			case 'up':
				$this->before_text = Scarcity_Samurai_GIF_Generator_Helper::get_param('before_text', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['up_counter_before_text']);
				break;
			case 'down':
				$this->before_text = Scarcity_Samurai_GIF_Generator_Helper::get_param('before_text', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['down_counter_before_text']);
				break;
		}

		if ($this->before_text === null) {
			$this->before_text_outer_width = 0;
		} else {
			$this->before_text .= ' ';
			$this->before_text_inner_width = self::text_width($this->before_text, $this->text_font, $this->text_font_size);
			$this->before_text_outer_width = $this->before_text_inner_width;
			$this->before_text_inner_height = self::text_height($this->before_text, $this->text_font, $this->text_font_size);
			$this->before_text_outer_height = $this->before_text_inner_height;
			$this->before_text_below_basepoint = self::below_basepoint($this->before_text, $this->text_font, $this->text_font_size);
		}

		// Text after counter
		switch ($counter_direction) {
			case 'up':
				$this->after_text = Scarcity_Samurai_GIF_Generator_Helper::get_param('after_text', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['up_counter_after_text']);
				break;
			case 'down':
				$this->after_text = Scarcity_Samurai_GIF_Generator_Helper::get_param('after_text', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['down_counter_after_text']);
				break;
		}

		if ($this->after_text === null) {
			$this->after_text_outer_width = 0;
		} else {
			$this->after_text = ' ' . $this->after_text;
			$this->after_text_inner_width = self::text_width($this->after_text, $this->text_font, $this->text_font_size);
			$this->after_text_outer_width = $this->after_text_inner_width;
			$this->after_text_inner_height = self::text_height($this->after_text, $this->text_font, $this->text_font_size);
			$this->after_text_outer_height = $this->after_text_inner_height;
			$this->after_text_below_basepoint = self::below_basepoint($this->after_text, $this->text_font, $this->text_font_size);
		}

		// Counter parameters
		$this->counter_font = Scarcity_Samurai_GIF_Generator_Helper::get_font_param('counter_font', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['counter_font']);
		$this->counter_font_size = Scarcity_Samurai_GIF_Generator_Helper::get_param('counter_font_size', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['counter_font_size']);
		$this->counter_color = Scarcity_Samurai_GIF_Generator_Helper::get_param('counter_color', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['counter_color']);

		// Expired message parameters
		$this->expired_text = Scarcity_Samurai_GIF_Generator_Helper::get_param('expired_text', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['expired_text']);
		$this->expired_font = Scarcity_Samurai_GIF_Generator_Helper::get_font_param('expired_font', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['expired_font']);
		$this->expired_font_size = Scarcity_Samurai_GIF_Generator_Helper::get_param('expired_font_size', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['expired_font_size']);
		$this->expired_color = Scarcity_Samurai_GIF_Generator_Helper::get_param('expired_color', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['expired_color']);

		if ($this->expired_text !== null) {
			$this->expired_outer_width = self::text_width($this->expired_text, $this->expired_font, $this->expired_font_size);
			$this->expired_outer_height = self::text_height($this->expired_text, $this->expired_font, $this->expired_font_size);
		}

		// Urgency message parameters
		$this->urgency_text = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_text', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['urgency_text']);
		$this->urgency_font = Scarcity_Samurai_GIF_Generator_Helper::get_font_param('urgency_font', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['urgency_font']);
		$this->urgency_font_size = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_font_size', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['urgency_font_size']);
		$this->urgency_color = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_color', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['urgency_color']);
		// Urgency message will appear when $this->age is:
 		//    $this->urgency_offset +     $this->urgency_frequency
		//    $this->urgency_offset + 2 * $this->urgency_frequency
		//    $this->urgency_offset + 3 * $this->urgency_frequency
		//             ...                            ...
		$this->urgency_offset = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_offset', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['urgency_offset']);
		$this->urgency_frequency = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_frequency', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['urgency_frequency']);
		// Urgency message will be visible for this number of seconds.
		// Set it to 0 to disable the urgency message.
		// Must be an integer.
		$this->urgency_delay = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_delay', $params, Scarcity_Samurai_GIF_Generator_Helper::$counter_defaults['urgency_delay']);

		// Other parameters

		// The number of frames in the GIF. Must be >= 1.
		$this->lifetime = 1;

		$this->build_frames();
	}

	protected function allocate_colors() {
		$this->image_background_color = $this->get_image_color('#ffffff');
		$this->text_color = $this->get_image_color($this->text_color);
		$this->counter_color = $this->get_image_color($this->counter_color);
		$this->counter_background_color = $this->image_background_color;
		$this->urgency_color = $this->get_image_color($this->urgency_color);
		$this->expired_color = $this->get_image_color($this->expired_color);
	}

	private function draw_aligned_text($text, $font, $font_size, $color,
	                                   $horizontal_alignment = 'center',
	                                   $vertical_alignment = 'center') {
		$this->clean_image();

		switch ($horizontal_alignment) {
			case 'left':
				$x = 0;
				break;
			case 'center':
				$text_width = self::text_width($text, $font, $font_size);
				$x = ($this->image_width - $text_width) / 2;
				break;
		}

		switch ($vertical_alignment) {
			case 'center':
				$text_height = self::text_height($text, $font, $font_size);
				$y = ($this->image_height - $text_height) / 2;
				break;
			case 'baseline':
				$text_below_basepoint = self::below_basepoint($text, $font, $font_size);
				$text_inner_height = self::text_height($text, $font, $font_size);
				$y = $this->baseline + $text_below_basepoint - $text_inner_height;
				break;
		}

		$this->draw_text($text, $font, $font_size,	$color,	$x,	$y);
	}

	private function add_frame() {
		if (($this->counter_expired === true) || ($this->age == $this->lifetime)) {
			if (($this->expired_text === null) || ($this->counter_expired === false)) {
				$this->expired = true;
				return;
			}

			if (isset($this->image)) {
				$this->draw_aligned_text($this->expired_text, $this->expired_font, $this->expired_font_size, $this->expired_color);
			} else {
				$this->image_width = $this->expired_outer_width;
				$this->image_height = $this->expired_outer_height;

				$this->create_image();
				$this->draw_aligned_text($this->expired_text, $this->expired_font, $this->expired_font_size, $this->expired_color);
			}

			$this->frame_time = 1;
			$this->expired = true;
		} else if (($this->urgency_delay > 0) && ($this->age > 0) &&
		           ($this->age - $this->urgency_offset) % $this->urgency_frequency == 0) {
			$this->draw_aligned_text($this->urgency_text, $this->urgency_font, $this->urgency_font_size, $this->urgency_color);
			$this->frame_time = $this->urgency_delay;
		} else {
			$counter_inner_width = self::text_width($this->counter_text, $this->counter_font, $this->counter_font_size);
			$counter_outer_width = $counter_inner_width;
			$counter_inner_height = self::text_height($this->counter_text, $this->counter_font, $this->counter_font_size);
			$counter_outer_height = $counter_inner_height;
			$counter_below_basepoint = self::below_basepoint($this->counter_text, $this->counter_font, $this->counter_font_size);

			$this->baseline = max($this->before_text_inner_height, $counter_inner_height, $this->after_text_inner_height);

			$this->image_width = $this->before_text_outer_width + $counter_outer_width + $this->after_text_outer_width;
			$this->image_height = max($this->before_text_outer_height, $counter_outer_height, $this->after_text_outer_height) +
			                      max($this->before_text_below_basepoint, $counter_below_basepoint, $this->after_text_below_basepoint);

			// On first frame, create an empty image.
			if ($this->age == 0) {
				$this->create_image();
			}

			$this->clean_image();

			// Draw the text before counter
			if ($this->before_text !== null) {
				$this->draw_text(
					$this->before_text,
					$this->text_font,
					$this->text_font_size,
					$this->text_color,
					0,
					$this->baseline + $this->before_text_below_basepoint - $this->before_text_inner_height
				);
			}

			// Draw counter background
			imagefilledrectangle($this->image, $this->before_text_outer_width, 0, $this->before_text_outer_width + $counter_outer_width - 1, $counter_outer_height - 1, $this->counter_background_color);

			// Draw the counter
			$this->draw_text(
				$this->counter_text,
				$this->counter_font,
				$this->counter_font_size,
				$this->counter_color,
				$this->before_text_outer_width,
				$this->baseline + $counter_below_basepoint - $counter_inner_height
			);

			// Draw the text after counter
			if ($this->after_text !== null) {
				$this->draw_text(
					$this->after_text,
					$this->text_font,
					$this->text_font_size,
					$this->text_color,
					$this->before_text_outer_width + $counter_outer_width,
					$this->baseline + $this->after_text_below_basepoint - $this->after_text_inner_height
				);
			}

			$this->frame_time = 1;
		}

		// Generate GIF from $this->image.
		// We want to put the binary GIF data into an array to be used later,
		// so we use the output buffer.
		ob_start();                                // Turn on output buffering
		imagegif($this->image);                    // Output the GIF to the buffer
		$this->frames[] = ob_get_clean();          // Get the current buffer contents and delete current output buffer
		$this->delays[] = 100 * $this->frame_time; // Delay in the animation. 100 === 1sec
	}

	private function build_frames() {
		while (!$this->expired && ($this->age <= $this->lifetime)) {
			$this->add_frame();
			$this->age += $this->frame_time;
		}

		if ($this->image !== null) {
			imagedestroy($this->image);
		}
	}

}
