<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

class Scarcity_Samurai_GIF_Generator_Helper {

	const FONTS_DIR = '../fonts/';

	static $timer_defaults = array();
	static $counter_defaults = array();

	public static function fonts() {
		$filenames = array();
		foreach ( glob(plugin_dir_path(__FILE__) . self::FONTS_DIR . '*.ttf') as $filename ) {
			$filenames[] = basename($filename, '.ttf');
		}
		return $filenames;
	}

	public static function set_timer_defaults() {
		// Text parameters
		self::$timer_defaults['text'] = null;
		self::$timer_defaults['text_font'] = 'Arial Bold';
		self::$timer_defaults['text_font_size'] = 28;
		self::$timer_defaults['text_padding_left'] = 0;
		self::$timer_defaults['text_padding_right'] = (int)(0.3 * self::$timer_defaults['text_font_size']);

		// Timer parameters
		self::$timer_defaults['timer_font'] = self::$timer_defaults['text_font'];
		self::$timer_defaults['timer_font_size'] = (int)(1.3 * self::$timer_defaults['text_font_size']);
		self::$timer_defaults['timer_color'] = '#000000';
		self::$timer_defaults['timer_padding_top'] = (int)(0.3 * self::$timer_defaults['timer_font_size']);
		self::$timer_defaults['timer_padding_right'] = (int)(0.4 * self::$timer_defaults['timer_font_size']);
		self::$timer_defaults['timer_padding_bottom'] = (int)(0.4 * self::$timer_defaults['timer_font_size']);
		self::$timer_defaults['timer_padding_left'] = self::$timer_defaults['timer_padding_right'];

		// Labels parameters
		self::$timer_defaults['labels_font'] = self::$timer_defaults['text_font'];
		self::$timer_defaults['labels_font_size'] = (int)(0.4 * self::$timer_defaults['text_font_size']);
		self::$timer_defaults['days_label'] = 'Days';
		self::$timer_defaults['hours_label'] = 'Hours';
		self::$timer_defaults['minutes_label'] = 'Minutes';
		self::$timer_defaults['seconds_label'] = 'Seconds';
		self::$timer_defaults['labels_padding_bottom'] = self::$timer_defaults['timer_padding_top'];

		// Urgency message parameters
		self::$timer_defaults['urgency_font'] = self::$timer_defaults['text_font'];
		self::$timer_defaults['urgency_font_size'] = (self::$timer_defaults['text'] === null ? (int)(0.4 * self::$timer_defaults['text_font']) : self::$timer_defaults['text_font']);
		self::$timer_defaults['urgency_message'] = 'Hurry Up - Time Is Running Out!';
		self::$timer_defaults['urgency_offset'] = 0;
		self::$timer_defaults['urgency_frequency'] = 10;
		self::$timer_defaults['urgency_delay'] = 3;

		// Expired message parameters
		self::$timer_defaults['expired_font'] = self::$timer_defaults['text_font'];
		self::$timer_defaults['expired_font_size'] = self::$timer_defaults['urgency_font_size'];
		self::$timer_defaults['expired_message'] = 'Sorry - This Offer Has Expired!';
		self::$timer_defaults['show_expired_message'] = true;

		// General parameters
		self::$timer_defaults['lifetime'] = 31;
	}

	public static function set_counter_defaults() {
		// Text parameters
		self::$counter_defaults['up_counter_before_text'] = 'Already';
		self::$counter_defaults['up_counter_after_text'] = 'item(s) sold';
		self::$counter_defaults['down_counter_before_text'] = 'Only';
		self::$counter_defaults['down_counter_after_text'] = 'item(s) left';
		self::$counter_defaults['text_font'] = 'Arial';
		self::$counter_defaults['text_font_size'] = 20;
		self::$counter_defaults['text_color'] = '#000000';

		// Counter parameters
		self::$counter_defaults['counter_font'] = 'Arial';
		self::$counter_defaults['counter_font_size'] = 20;
		self::$counter_defaults['counter_color'] = '#000000';

		// Urgency message parameters
		self::$counter_defaults['urgency_text'] = 'Hurry Up - Time Is Running Out!';
		self::$counter_defaults['urgency_font'] = 'Arial';
		self::$counter_defaults['urgency_font_size'] = 20;
		self::$counter_defaults['urgency_color'] = '#ff0000';
		self::$counter_defaults['urgency_offset'] = 0;
		self::$counter_defaults['urgency_frequency'] = 3;
		self::$counter_defaults['urgency_delay'] = 1;

		// Expired message parameters
		self::$counter_defaults['expired_text'] = 'Sorry - This Offer Has Expired!';
		self::$counter_defaults['expired_font'] = 'Arial Bold';
		self::$counter_defaults['expired_font_size'] = 20;
		self::$counter_defaults['expired_color'] = '#ff0000';
	}

	// Source: http://www.php.net/manual/en/function.imagettfbbox.php#75491
	public static function convert_bounding_box($box) {
		if ($box[0] >= -1) {
			$x_offset = -abs($box[0] + 1);
		}	else {
			$x_offset = abs($box[0] + 2);
		}

		$width = abs($box[2] - $box[0]);

		if ($box[0] < -1) {
			$width = abs($box[2]) + abs($box[0]) - 1;
		}

		$y_offset = abs($box[5] + 1);

		if ($box[5] >= -1) {
			$y_offset = -$y_offset; // Fixed characters below the baseline.
		}

		$height = abs($box[7]) - abs($box[1]);

		if ($box[3] > 0) {
			$height = abs($box[7] - $box[1]) - 1;
		}

		return array(
			'width' => $width,
			'height' => $height,
			'x_offset' => $x_offset,
			'y_offset' => $y_offset,
			'below_basepoint' => max(0, $box[1])
		);
	}

	private static function exact_text_width($text, $font, $size) {
		$box = imagettfbbox($size, 0, $font, $text);
		$converted = self::convert_bounding_box($box);
		return $converted['width'];
	}

	public static function text_width($text, $font, $size) {
		return self::exact_text_width('w' . $text . 'w', $font, $size) -
		       self::exact_text_width('ww', $font, $size);
	}

	public static function text_height($text, $font, $size) {
		$box = imagettfbbox($size, 0, $font, $text);
		$converted = self::convert_bounding_box($box);
		return $converted['height'];
	}

	public static function below_basepoint($text, $font, $size) {
		$box = imagettfbbox($size, 0, $font, $text);
		$converted = self::convert_bounding_box($box);
		return $converted['below_basepoint'];
	}

	public static function draw_text($image, $text, $font, $size, $color, $x, $y) {
		$box = imagettfbbox($size, 0, $font, $text);
		$converted = self::convert_bounding_box($box);
		imagettftext($image, $size, 0, $x + $converted['x_offset'], $y + $converted['y_offset'], $color, $font, $text);
	}

	// Encodes the specified params to a string that will be sent in URL.
	public static function encode_params($params) {
		return base64_encode(json_encode($params));
	}

	// Decodes the specified URL params string into a PHP associative array.
	// On error, returns null.
	public static function decode_params($params) {
		$params = base64_decode($params);

		if ($params === false) {
			return null;
		}

		$params = json_decode($params, true);

		if ($params === null) {
			return null;
		}

		return $params;
	}

	// Returns a link to timer GIF with the specified parameters.
	public static function timer_link($params) {
		// Encode the parameters
		$params = 'params=' . self::encode_params($params);

		// Build the link
		return self::url('endpoints/timer-gif.php') . "?$params";
	}

	// Returns a link to counter GIF with the specified parameters.
	public static function counter_link($params) {
		// Encode the parameters
		$params = 'params=' . self::encode_params($params);

		// Build the link
		return Scarcity_Samurai_Helper::url('endpoints/counter-gif.php') . "?$params";
	}

	public static function get_param($key, $params, $default_value = null) {
		return array_key_exists($key, $params) ? $params[$key] : $default_value;
	}

	public static function get_font_param($key, $params, $default_font) {
		return self::FONTS_DIR . self::get_param($key, $params, $default_font) . '.ttf';
	}

}

Scarcity_Samurai_GIF_Generator_Helper::set_timer_defaults();
Scarcity_Samurai_GIF_Generator_Helper::set_counter_defaults();
