<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

require_once('../vendor/gif-encoder/class-gif-encoder.php');
require_once('gif-generator-helper.php');

class Scarcity_Samurai_GIF_Generator {

	function __construct($params) {
		$this->params = $params;

		$this->lifetime = 1;
		$this->age = 0;
		$this->expired = false;

		$this->frames = array();
		$this->delays = array();

		$this->image_width = null;
		$this->image_height = null;
		$this->image = null;
	}

	protected function clean_image() {
		imagefilledrectangle($this->image, 0, 0, $this->image_width - 1, $this->image_height - 1, $this->image_background_color);
	}

	protected function create_image() {
		$this->image = imagecreatetruecolor($this->image_width, $this->image_height);
		$this->allocate_colors();
	}

	protected function get_image_color($color) {
		$r = hexdec(substr($color, 1, 2));
		$g = hexdec(substr($color, 3, 2));
		$b = hexdec(substr($color, 5, 2));

		return imagecolorallocate($this->image, $r, $g, $b);
	}

		// Source: http://www.php.net/manual/en/function.imagettfbbox.php#75491
	private static function convert_bounding_box($box) {
		if ($box[0] >= -1) {
			$x_offset = -abs($box[0] + 1);
		}	else {
			$x_offset = abs($box[0] + 2);
		}

		$width = abs($box[2] - $box[0]);

		if ($box[0] < -1) {
			$width = abs($box[2]) + abs($box[0]) - 1;
		}

		$y_offset = abs($box[5] + 1);

		if ($box[5] >= -1) {
			$y_offset = -$y_offset; // Fixed characters below the baseline.
		}

		$height = abs($box[7]) - abs($box[1]);

		if ($box[3] > 0) {
			$height = abs($box[7] - $box[1]) - 1;
		}

		return array(
			'width' => $width,
			'height' => $height,
			'x_offset' => $x_offset,
			'y_offset' => $y_offset,
			'below_basepoint' => max(0, $box[1])
		);
	}

	private static function exact_text_width($text, $font, $size) {
		$box = imagettfbbox($size, 0, $font, $text);
		$converted = self::convert_bounding_box($box);
		return $converted['width'];
	}

	protected static function text_width($text, $font, $size) {
		return self::exact_text_width('w' . $text . 'w', $font, $size) -
		       self::exact_text_width('ww', $font, $size);
	}

	protected static function text_height($text, $font, $size) {
		$box = imagettfbbox($size, 0, $font, $text);
		$converted = self::convert_bounding_box($box);
		return $converted['height'];
	}

	protected static function below_basepoint($text, $font, $size) {
		$box = imagettfbbox($size, 0, $font, $text);
		$converted = self::convert_bounding_box($box);
		return $converted['below_basepoint'];
	}

	protected function draw_text($text, $font, $size, $color, $x, $y) {
		$box = imagettfbbox($size, 0, $font, $text);
		$converted = self::convert_bounding_box($box);
		imagettftext($this->image, $size, 0, $x + $converted['x_offset'], $y + $converted['y_offset'], $color, $font, $text);
	}

	public function get_gif() {
		$gif = new GIFEncoder($this->frames, $this->delays, -1, 2, 0, 0, 0, array(), 'bin');
		return $gif->GetAnimation();
	}

	public function is_empty() {
		return (count($this->frames) === 0);
	}

}
