<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

require_once('gif-generator.php');

class Scarcity_Samurai_Timer_GIF_Generator extends Scarcity_Samurai_GIF_Generator {

	function __construct($params) {
		parent::__construct($params);

		// Text parameters
		$this->text = Scarcity_Samurai_GIF_Generator_Helper::get_param('text', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['text']);
		$this->text_font = Scarcity_Samurai_GIF_Generator_Helper::get_font_param('text_font', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['text_font']);
		$this->text_font_size = Scarcity_Samurai_GIF_Generator_Helper::get_param('text_font_size', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['text_font_size']);
		$this->text_padding_left = Scarcity_Samurai_GIF_Generator_Helper::get_param('text_padding_left', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['text_padding_left']);
		$this->text_padding_right = Scarcity_Samurai_GIF_Generator_Helper::get_param('text_padding_right', $params, (int)(0.3 * $this->text_font_size));

		// Timer parameters
		$this->timer_font = Scarcity_Samurai_GIF_Generator_Helper::get_param('timer_font', $params, $this->text_font); // Not 'get_font_param()'
		$this->timer_font_size = Scarcity_Samurai_GIF_Generator_Helper::get_param('timer_font_size', $params, (int)(1.3 * $this->text_font_size));
		$this->timer_color = Scarcity_Samurai_GIF_Generator_Helper::get_param('timer_color', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['timer_color']);
		$this->timer_padding_top = Scarcity_Samurai_GIF_Generator_Helper::get_param('timer_padding_top', $params, (int)(0.3 * $this->timer_font_size));
		$this->timer_padding_right = Scarcity_Samurai_GIF_Generator_Helper::get_param('timer_padding_right', $params, (int)(0.4 * $this->timer_font_size));
		$this->timer_padding_bottom = Scarcity_Samurai_GIF_Generator_Helper::get_param('timer_padding_bottom', $params, (int)(0.4 * $this->timer_font_size));
		$this->timer_padding_left = Scarcity_Samurai_GIF_Generator_Helper::get_param('timer_padding_left', $params, $this->timer_padding_right);

		// Labels parameters
		$this->labels_font = Scarcity_Samurai_GIF_Generator_Helper::get_param('labels_font', $params, $this->text_font); // Not 'get_font_param()'
		$this->labels_font_size = Scarcity_Samurai_GIF_Generator_Helper::get_param('labels_font_size', $params, (int)(0.4 * $this->text_font_size));
		$this->days_label = Scarcity_Samurai_GIF_Generator_Helper::get_param('days_label', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['days_label']);
		$this->hours_label = Scarcity_Samurai_GIF_Generator_Helper::get_param('hours_label', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['hours_label']);
		$this->minutes_label = Scarcity_Samurai_GIF_Generator_Helper::get_param('minutes_label', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['minutes_label']);
		$this->seconds_label = Scarcity_Samurai_GIF_Generator_Helper::get_param('seconds_label', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['seconds_label']);
		$this->labels_padding_bottom = Scarcity_Samurai_GIF_Generator_Helper::get_param('labels_padding_bottom', $params, $this->timer_padding_top);
		$this->labels_height = Scarcity_Samurai_GIF_Generator_Helper::text_height(
			$this->days_label . $this->hours_label . $this->minutes_label . $this->seconds_label,
			$this->labels_font,
			$this->labels_font_size
		);

		// Urgency message parameters
		$this->urgency_font = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_font', $params, $this->text_font); // Not 'get_font_param()'
		$this->urgency_font_size = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_font_size', $params, $this->text === null ? (int)(0.4 * $this->text_font_size) : $this->text_font_size);
		$this->urgency_message = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_message', $params, 'Hurry Up - Time Is Running Out!');
		// Urgency message will appear when $this->age is:
 		//    $this->urgency_offset +     $this->urgency_frequency
		//    $this->urgency_offset + 2 * $this->urgency_frequency
		//    $this->urgency_offset + 3 * $this->urgency_frequency
		//             ...                            ...
		$this->urgency_offset = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_offset', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['urgency_offset']);
		$this->urgency_frequency = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_frequency', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['urgency_frequency']);
		// Urgency message will be visible for this number of seconds.
		// Set it to 0 to disable the urgency message.
		// Must be an integer.
		$this->urgency_delay = Scarcity_Samurai_GIF_Generator_Helper::get_param('urgency_delay', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['urgency_delay']);

		// Expired message parameters
		$this->expired_font = Scarcity_Samurai_GIF_Generator_Helper::get_param('expired_font', $params, $this->text_font); // Not 'get_font_param()'
		$this->expired_font_size = Scarcity_Samurai_GIF_Generator_Helper::get_param('expired_font_size', $params, $this->urgency_font_size);
		$this->expired_message = Scarcity_Samurai_GIF_Generator_Helper::get_param('expired_message', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['expired_message']);
		$this->show_expired_message = Scarcity_Samurai_GIF_Generator_Helper::get_param('show_expired_message', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['show_expired_message']);

		if ($this->show_expired_message) {
			$this->expired_inner_width = Scarcity_Samurai_GIF_Generator_Helper::text_width($this->expired_message, $this->expired_font, $this->expired_font_size);
			$this->expired_outer_width = $this->expired_inner_width;
			$this->expired_inner_height = Scarcity_Samurai_GIF_Generator_Helper::text_height($this->expired_message, $this->expired_font, $this->expired_font_size);
			$this->expired_outer_height = $this->expired_inner_height;
		} else {
			$this->expired_outer_width = 0;
			$this->expired_outer_height = 0;
		}

		// Other parameters

		// The number of frames in the GIF. Must be >= 1.
		$this->lifetime = Scarcity_Samurai_GIF_Generator_Helper::get_param('lifetime', $params, Scarcity_Samurai_GIF_Generator_Helper::$timer_defaults['lifetime']);

		$this->timer_value = Scarcity_Samurai_GIF_Generator_Helper::get_param('timer_value', $params, 20);

		if ($this->text === null) {
			$this->text_outer_width = 0;
		} else {
			$this->text_inner_width = Scarcity_Samurai_GIF_Generator_Helper::text_width($this->text, $this->text_font, $this->text_font_size);
			$this->text_outer_width = $this->text_padding_left + $this->text_inner_width + $this->text_padding_right;
			$this->text_inner_height = Scarcity_Samurai_GIF_Generator_Helper::text_height($this->text, $this->text_font, $this->text_font_size);
			$this->text_below_basepoint = Scarcity_Samurai_GIF_Generator_Helper::below_basepoint($this->text, $this->text_font, $this->text_font_size);
		}

		$this->build_frames();
	}

	protected function allocate_colors() {
		$this->image_background_color = $this->get_image_color('#ffffff');
		$this->text_color = $this->get_image_color('#000000');
		$this->timer_background_color = $this->get_image_color('#ffffff');
		$this->timer_color = $this->get_image_color($this->timer_color);
		$this->labels_color = $this->get_image_color('#000000');
		$this->urgency_color = $this->get_image_color('#ff0000');
		$this->expired_color = $this->urgency_color;
	}

	private function draw_label_text($text, $x, $y) {
		Scarcity_Samurai_GIF_Generator_Helper::draw_text(
			$this->image,
			$text,
			$this->labels_font,
			$this->labels_font_size,
			$this->labels_color,
			$x,
			$y
		);
	}

	private function draw_aligned_text($text, $font, $font_size, $color,
	                                   $horizontal_alignment = 'center',
	                                   $vertical_alignment = 'center') {
		$this->clean_image();

		switch ($horizontal_alignment) {
			case 'left':
				$x = $this->text_padding_left;
				break;
			case 'center':
				$text_width = self::text_width($text, $font, $font_size);
				$x = ($this->image_width - $text_width) / 2;
				break;
		}

		switch ($vertical_alignment) {
			case 'center':
				$text_height = self::text_height($text, $font, $font_size);
				$y = ($this->image_height - $text_height) / 2;
				break;
			case 'baseline':
				$text_below_basepoint = self::below_basepoint($text, $font, $font_size);
				$text_inner_height = self::text_height($text, $font, $font_size);
				$y = $this->baseline + $text_below_basepoint - $text_inner_height;
				break;
		}

		$this->draw_text($text, $font, $font_size,	$color,	$x,	$y);
	}

	private function add_frame() {
		if (($this->timer_value <= 0) || ($this->age == $this->lifetime)) {
			if (!$this->show_expired_message || ($this->timer_value > 0)) {
				return;
			}

			if (isset($this->image)) {
				$this->draw_aligned_text($this->expired_message, $this->expired_font, $this->expired_font_size, $this->expired_color);
			} else {
				$this->image_width = $this->expired_outer_width;
				$this->image_height = $this->expired_outer_height;
				$this->create_image();
				$this->draw_aligned_text($this->expired_message, $this->expired_font, $this->expired_font_size, $this->expired_color);
			}

			$this->frame_time = 1;
			$this->expired = true;
		} else if (($this->urgency_delay > 0) && ($this->age > 0) &&
		           ($this->age - $this->urgency_offset) % $this->urgency_frequency == 0) {
			$this->draw_aligned_text($this->urgency_message, $this->urgency_font, $this->urgency_font_size, $this->urgency_color);
			$this->frame_time = $this->urgency_delay;
		} else {
			// Prepare timer
			$this->prepare_timer();
			$timer_text = $this->format_timer();

			$timer_inner_width = self::text_width($timer_text, $this->timer_font, $this->timer_font_size);
			$timer_outer_width = $this->timer_padding_left + $timer_inner_width + $this->timer_padding_right;
			$timer_inner_height = self::text_height($timer_text, $this->timer_font, $this->timer_font_size);
			$timer_outer_height = $this->timer_padding_top + $timer_inner_height + $this->timer_padding_bottom;

			$this->baseline = $this->timer_padding_top + $timer_inner_height;

			$this->image_width = $this->text_outer_width + $timer_outer_width;
			$this->image_height = $timer_outer_height + $this->labels_height + $this->labels_padding_bottom;

			// On first frame, create an empty image.
			if ($this->age == 0) {
				$this->create_image();
			}

			// Clean the image with timer's background color
			$this->clean_image();

			// Draw text
			if ($this->text !== null) {
				$this->draw_text(
					$this->text,
					$this->text_font,
					$this->text_font_size,
					$this->text_color,
					$this->text_padding_left,
					$this->baseline + $this->text_below_basepoint - $this->text_inner_height
				);
			}

			// Draw timer background
			imagefilledrectangle($this->image, $this->text_outer_width, 0, $this->text_outer_width + $timer_outer_width - 1, $timer_outer_height - 1, $this->timer_background_color);

			// Draw timer
			$this->draw_text(
				$timer_text,
				$this->timer_font,
				$this->timer_font_size,
				$this->timer_color,
				$this->text_outer_width + $this->timer_padding_left,
				$this->timer_padding_top
			);

			// Draw labels
			if ($this->days > 0) {
				$text_width = self::text_width($this->format_timer('d', true), $this->timer_font, $this->timer_font_size);
				$days_x_middle = $text_width / 2;
				$days_label_width = self::text_width($this->days_label, $this->labels_font, $this->labels_font_size);
				$days_label_x = $days_x_middle - $days_label_width / 2;
			}

			$short_text_width = self::text_width($this->format_timer('d:'), $this->timer_font, $this->timer_font_size);
			$long_text_width = self::text_width($this->format_timer('d:hh'), $this->timer_font, $this->timer_font_size);
			$hours_x_middle = ($short_text_width + $long_text_width) / 2;
			$hours_label_width = self::text_width($this->hours_label, $this->labels_font, $this->labels_font_size);
			$hours_label_x = $hours_x_middle - $hours_label_width / 2;

			$short_text_width = self::text_width($this->format_timer('d:hh:'), $this->timer_font, $this->timer_font_size);
			$long_text_width = self::text_width($this->format_timer('d:hh:mm'), $this->timer_font, $this->timer_font_size);
			$minutes_x_middle = ($short_text_width + $long_text_width) / 2;
			$minutes_label_width = self::text_width($this->minutes_label, $this->labels_font, $this->labels_font_size);
			$minutes_label_x = $minutes_x_middle - $minutes_label_width / 2;

			$short_text_width = self::text_width($this->format_timer('d:hh:mm:'), $this->timer_font, $this->timer_font_size);
			$long_text_width = self::text_width($this->format_timer('d:hh:mm:ss'), $this->timer_font, $this->timer_font_size);
			$seconds_x_middle = ($short_text_width + $long_text_width) / 2;
			$seconds_label_width = self::text_width($this->seconds_label, $this->labels_font, $this->labels_font_size);
			$seconds_label_x = $seconds_x_middle - $seconds_label_width / 2;

			if ($this->days > 0) {
				$this->draw_label_text('Days', $this->text_outer_width + $this->timer_padding_left + $days_label_x, $timer_outer_height);
			}

			$this->draw_label_text('Hours', $this->text_outer_width + $this->timer_padding_left + $hours_label_x, $timer_outer_height);
			$this->draw_label_text('Minutes', $this->text_outer_width + $this->timer_padding_left + $minutes_label_x, $timer_outer_height);
			$this->draw_label_text('Seconds', $this->text_outer_width + $this->timer_padding_left + $seconds_label_x, $timer_outer_height);

			$this->frame_time = 1;
		}

		// Generate GIF from $this->image.
		// We want to put the binary GIF data into an array to be used later,
		// so we use the output buffer.
		ob_start();                                // Turn on output buffering
		imagegif($this->image);                    // Output the GIF to the buffer
		$this->frames[] = ob_get_clean();          // Get the current buffer contents and delete current output buffer
		$this->delays[] = 100 * $this->frame_time; // Delay in the animation. 100 === 1sec
	}

	private function build_frames() {
		while (!$this->expired && ($this->age <= $this->lifetime)) {
			$this->add_frame();
			$this->age += $this->frame_time;
			$this->timer_value -= $this->frame_time;
		}

		if ($this->image !== null) {
			imagedestroy($this->image);
		}
	}

	private function prepare_timer() {
		$this->seconds = $this->timer_value;

		$this->days = (int)($this->seconds / 86400);
		$this->seconds -= $this->days * 86400;

		$this->hours = (int)($this->seconds / 3600);
		$this->seconds -= $this->hours * 3600;

		$this->minutes = (int)($this->seconds / 60);
		$this->seconds -= $this->minutes * 60;
	}

	private function format_timer($format = 'd:hh:mm:ss', $always_show_days = false) {
		if ($always_show_days) {
			return str_replace(
				array('d', 'hh', 'mm', 'ss'),
				array(
					sprintf('%d', $this->days),
					sprintf('%02d', $this->hours),
					sprintf('%02d', $this->minutes),
					sprintf('%02d', $this->seconds)),
				$format
			);
		} else {
			return str_replace(
				array('d:', 'hh', 'mm', 'ss'),
				array(
					($this->days == 0 ? '' : sprintf('%d:', $this->days)),
					sprintf('%02d', $this->hours),
					sprintf('%02d', $this->minutes),
					sprintf('%02d', $this->seconds)),
				$format
			);
		}
	}

}
