<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

add_action( 'plugins_loaded', array( 'Scarcity_Samurai_Inline_Banners', 'init' ) );

class Scarcity_Samurai_Inline_Banners {

	private static $current_page = null;
	private static $campaign_id = null;
	public static $html_comment_regex = '/<!--\s*ss-banner\s+(.*?)\s*-->/';

	public static function init() {
		add_action( 'admin_head', array( __CLASS__, 'admin_head' ) );
		// Priority 11 means that the 'Add Banner' button should appear AFTER
		// the 'Add Media' button.
		add_action( 'media_buttons', array( __CLASS__, 'add_media_buttons'), 11 );

		if ( get_user_option('rich_editing') ) { // Make sure that TinyMCE is used
			add_filter( 'mce_external_plugins', array( 'Scarcity_Samurai_Inline_Banners', 'add_tinymce_plugins' ) );
		}
	}

	public static function admin_head() {
		self::$current_page = Scarcity_Samurai_Helper::current_page();
		self::$campaign_id = self::$current_page['campaign_id'];
	}

	public static function admin_footer() {
		global $scarcity_samurai_dir;

		wp_enqueue_style( 'ss-jquery-ui', Scarcity_Samurai::$JQUERY_UI_CSS_URL );
		wp_enqueue_script( 'ss-helper', Scarcity_Samurai_Helper::url( 'scripts/js/helper.js' ) );
		wp_enqueue_script( 'ss-inline-banners', Scarcity_Samurai_Helper::url( 'scripts/js/dialogs/add-inline-banner.js' ) );

		Scarcity_Samurai_Helper::echo_html( $scarcity_samurai_dir . 'html/dialogs/add-inline-banner.php', array(
			'campaign_id' => self::$campaign_id
		) );
	}

	public static function add_media_buttons() {
		global $scarcity_samurai_dir;
		add_action( 'admin_footer', array( __CLASS__, 'admin_footer' ) );

		$has_inline_banners = Scarcity_Samurai_Page::has_inline_banners();
		$display_no_campaign_note = false;
		$display_no_timer_note = false;
		if ( $has_inline_banners ) {
			$display_no_campaign_note = ( self::$campaign_id === null );

			$available_until = self::$current_page['available_until'];
			$display_no_timer_note = ! $display_no_campaign_note && ( ! isset( $available_until['enabled'] ) || ( $available_until['enabled'] === false ) );
		}

		include( $scarcity_samurai_dir . 'html/add-inline-banner-button.php' );
	}

	public static function add_tinymce_plugins( $plugins ) {
		$plugins['ss_inline_banner'] = Scarcity_Samurai_Helper::url( 'scripts/js/tinymce/inline_banner.js' );

		return $plugins;
	}

}
