<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

class Scarcity_Samurai_Counter extends Scarcity_Samurai_Model {

	public $short_name, $table_name;
	protected $fields;

	public function __construct() {
		global $wpdb;

		$this->short_name = 'counter';
		$this->table_name = $wpdb->prefix . 'scarcity_samurai_counters';

		$this->fields = array(
			'id' => array(
				'type' => 'string'
			),
			'name' => array(
				'type' => 'string'
			),
			'value' => array(
				'type' => 'integer'
			),
			'data' => array(
				'type' => 'json'
			)
		);
	}

	public static function on_activate() {
		global $wpdb, $scarcity_samurai_charset_collate;

		self::$sql = "
			CREATE TABLE IF NOT EXISTS " . Scarcity_Samurai_Model::get( 'Counter' )->table_name . " (
				id VARCHAR(32) NOT NULL,
				name VARCHAR(100) NOT NULL,
				value BIGINT(20) UNSIGNED NOT NULL,
				data LONGTEXT,
				PRIMARY KEY (id)
			) $scarcity_samurai_charset_collate";

		$wpdb->query(self::$sql);
	}

	public function delete($data) {
		// Firstly, delete all the counters access records.
		if (Scarcity_Samurai_Model::get( 'Counters_Access' )->delete(array('counter_id' => $data['id'])) === false) {
			return false;
		}

		// Then, delete the counters themselves.
		return parent::delete(array('id' => $data['id']));
	}

	public static function limited($id) {
		$counter = Scarcity_Samurai_Model::get( 'Counter' )->find($id);

		switch ($counter['data']['direction']) {
			case 'up':
				return array_key_exists('max', $counter['data']);

			case 'down':
				return array_key_exists('min', $counter['data']);
		}

		return null;
	}

	public static function expired($id) {
		$counter = Scarcity_Samurai_Model::get( 'Counter' )->get_model($id);

		return (($counter['data'] !== null) &&
		        array_key_exists('max', $counter['data']) &&
		        ($counter['value'] >= $counter['data']['max'] - $counter['data']['min']));
	}

	public static function display_value($id) {
		$counter = Scarcity_Samurai_Model::get( 'Counter' )->get_model($id);

		switch ($counter['data']['direction']) {
			case 'up':
				if (array_key_exists('max', $counter['data'])) {
					return min($counter['data']['min'] + $counter['value'], $counter['data']['max']);
				}

				return $counter['data']['min'] + $counter['value'];

			case 'down':
				return max($counter['data']['max'] - $counter['value'], $counter['data']['min']);
		}

		return null;
	}

	public static function get_display_value_by_id($id, $show_from) {
		$counter = Scarcity_Samurai_Model::get( 'Counter' )->find($id);

		if ($counter !== null) {
			$direction = $counter['data']['direction'];
			$display_value = self::display_value($counter);

			if (($show_from === null) ||
			    ((($direction === 'up') && ($display_value >= $show_from)) ||
			     (($direction === 'down') && ($display_value <= $show_from)))) {
				return $display_value;
			}
		}

		return '';
	}

}
