<?php

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

class Scarcity_Samurai_Page_Meta_Box {

	public static function save_page_meta_box_data($page_id) {
		global $wpdb;

		// Create the Scarcity Samurai page if it doesn't exist yet.
		$page = Scarcity_Samurai_Model::get( 'Page' )->find($page_id);

		if ($page === null) {
			$page = Scarcity_Samurai_Model::get( 'Page' )->insert(array(
				'id' => $page_id,
				'type_id' => Scarcity_Samurai_Page_Type::other_page_type_id(),
				'position' => 0
			));
		}

		// If user just clicked 'Add New' (page), do nothing.
		if (!isset($_POST['ss-page-campaign'])) {
			return;
		}

		// If user updated the page with 'No Campaign' selected, set 'campaign_id' to null.
		if ($_POST['ss-page-campaign'] === '') {
			if (!Scarcity_Samurai_Model::get( 'Page' )->update(array('campaign_id' => null), $wpdb->prepare('id = %d', $page_id))) {
				Scarcity_Samurai_Helper::error('Page update failed!');
			}

			return;
		}

		$campaign_id = intval($_POST['ss-page-campaign']);
		$type_id = intval($_POST['ss-page-type-id']);

		// ----------------------------
		// ---| Access Restriction |---
		// ----------------------------

		$available_from = array(
			'enabled' => (array_key_exists('ss-page-lock-enable', $_POST) &&
			              ($_POST['ss-page-lock-enable'] === 'on')),
			'type' => $_POST['ss-page-lock-from']
		);

		// Visitors can access this page...
		switch ($_POST['ss-page-lock-from']) {
			case 'opt_in':
				if ($_POST['ss-page-lock-from-optin-page-id'] !== '') {
					$available_from['page_id'] = intval($_POST['ss-page-lock-from-optin-page-id']);
				}

				$available_from['value'] = Scarcity_Samurai_Helper::calculate_time_period(array(
					'days' => $_POST['ss-page-lock-from-optin-days'],
					'hours' => $_POST['ss-page-lock-from-optin-hours'],
					'minutes' => $_POST['ss-page-lock-from-optin-minutes'],
					'seconds' => $_POST['ss-page-lock-from-optin-seconds']
				));

				break;

			case 'fixed':
				$available_from['timezone'] = $_POST['ss-page-lock-from-fixed-timezone'];
				$available_from['value'] = Scarcity_Samurai_Helper::calculate_fixed_time(array(
					'year' => $_POST['ss-page-lock-from-fixed-year'],
					'month' => $_POST['ss-page-lock-from-fixed-month'],
					'day' => $_POST['ss-page-lock-from-fixed-day'],
					'hour' => $_POST['ss-page-lock-from-fixed-hour'],
					'minute' => $_POST['ss-page-lock-from-fixed-minute'],
					'timezone' => $_POST['ss-page-timer-from-fixed-timezone']
				));

				break;
		}

		// If a visitor attempts to access this page before it is available...
		switch ($_POST['ss-page-lock-early']) {
			case 'page_not_found':
				$available_from['too_early_action'] = array(
					'error' => 404
				);

				break;

			case 'redirect_to_page':
				if ($_POST['ss-page-lock-early-redirect-page-id'] === '') {
					$available_from['too_early_action'] = array(
						'error' => 404
					);
				} else {
					$available_from['too_early_action'] = array(
						'redirect' => array(
							'page_id' => intval($_POST['ss-page-lock-early-redirect-page-id'])
						)
					);
				}

				break;

			case 'redirect_to_url':
				$redirect_url = trim($_POST['ss-page-lock-early-redirect-url']);

				if ($redirect_url === '') {
					$available_from['too_early_action'] = array(
						'error' => 404
					);
				} else {
					$available_from['too_early_action'] = array(
						'redirect' => array(
							'url' => Scarcity_Samurai_Helper::add_http_to_url_if_required($redirect_url)
						)
					);
				}

				break;
		}

		// If a visitor has not opted-in...
		if ($_POST['ss-page-lock-from'] === 'opt_in') {
			switch ($_POST['ss-page-lock-not-opted-in']) {
				case 'page_not_found':
					$available_from['not_opted_in_action'] = array(
						'error' => 404
					);

					break;

				case 'redirect_to_page':
					if ($_POST['ss-page-lock-not-opted-in-redirect-page-id'] === '') {
						$available_from['not_opted_in_action'] = array(
							'error' => 404
						);
					} else {
						$available_from['not_opted_in_action'] = array(
							'redirect' => array(
								'page_id' => intval($_POST['ss-page-lock-not-opted-in-redirect-page-id'])
							)
						);
					}

					break;

				case 'redirect_to_url':
					$redirect_url = trim($_POST['ss-page-lock-not-opted-in-redirect-url']);

					if ($redirect_url === '') {
						$available_from['not_opted_in_action'] = array(
							'error' => 404
						);
					} else {
						$available_from['not_opted_in_action'] = array(
							'redirect' => array(
								'url' => Scarcity_Samurai_Helper::add_http_to_url_if_required($redirect_url)
							)
						);
					}

					break;
			}
		}

		// --------------------------
		// ---| Count Down Timer |---
		// --------------------------

		$available_until = array(
			'enabled' => (array_key_exists('ss-page-timer-enable', $_POST) &&
			              ($_POST['ss-page-timer-enable'] === 'on')),
			'type' => $_POST['ss-page-timer-from']
		);

		// Count down relative to...
		switch ($_POST['ss-page-timer-from']) {
			case 'page_load':
				$available_until['value'] = Scarcity_Samurai_Helper::calculate_time_period(array(
					'days' => $_POST['ss-page-timer-from-page-load-days'],
					'hours' => $_POST['ss-page-timer-from-page-load-hours'],
					'minutes' => $_POST['ss-page-timer-from-page-load-minutes'],
					'seconds' => $_POST['ss-page-timer-from-page-load-seconds']
				));

				break;

			case 'opt_in':
				if ($_POST['ss-page-timer-from-optin-page-id'] !== '') {
					$available_until['page_id'] = intval($_POST['ss-page-timer-from-optin-page-id']);
				}

				$available_until['value'] = Scarcity_Samurai_Helper::calculate_time_period(array(
					'days' => $_POST['ss-page-timer-from-optin-days'],
					'hours' => $_POST['ss-page-timer-from-optin-hours'],
					'minutes' => $_POST['ss-page-timer-from-optin-minutes'],
					'seconds' => $_POST['ss-page-timer-from-optin-seconds']
				));

				break;

			case 'fixed':
				$available_until['timezone'] = $_POST['ss-page-timer-from-fixed-timezone'];
				$available_until['value'] = Scarcity_Samurai_Helper::calculate_fixed_time(array(
					'year' => $_POST['ss-page-timer-from-fixed-year'],
					'month' => $_POST['ss-page-timer-from-fixed-month'],
					'day' => $_POST['ss-page-timer-from-fixed-day'],
					'hour' => $_POST['ss-page-timer-from-fixed-hour'],
					'minute' => $_POST['ss-page-timer-from-fixed-minute'],
					'timezone' => $_POST['ss-page-timer-from-fixed-timezone']
				));

				break;
		}

		// When the timer reaches zero...
		switch ($_POST['ss-page-timer-expired']) {
			case 'do_nothing':
				$available_until['too_late_action'] = array(
					'do_nothing' => true
				);

				break;

			case 'redirect_to_page':
				if ($_POST['ss-page-timer-expired-redirect-page-id'] === '') {
					$available_until['too_late_action'] = array(
						'do_nothing' => true
					);
				} else {
					$available_until['too_late_action'] = array(
						'redirect' => array(
							'page_id' => intval($_POST['ss-page-timer-expired-redirect-page-id'])
						)
					);
				}

				break;

			case 'redirect_to_url':
				$redirect_url = trim($_POST['ss-page-timer-expired-redirect-url']);

				if ($redirect_url === '') {
					$available_until['too_late_action'] = array(
						'do_nothing' => true
					);
				} else {
					$available_until['too_late_action'] = array(
						'redirect' => array(
							'url' => Scarcity_Samurai_Helper::add_http_to_url_if_required($redirect_url)
						)
					);
				}

				break;
		}

		// ----------------------
		// ---| Item Counter |---
		// ----------------------

		$counter = array(
			'enabled' => (isset($_POST['ss-page-counter-enable']) && ($_POST['ss-page-counter-enable'] === 'on')),
			'counter_id' => isset($_POST['ss-page-counter-counter-id']) ? $_POST['ss-page-counter-counter-id'] : null
		);

		// Show banners containing a counter...
		if (isset($_POST['ss-page-counter-show-when']) && ($_POST['ss-page-counter-show-when'] === 'reached_show_from')) {
			$counter['show_from'] = intval($_POST['ss-page-counter-show-from']);
		}

		// When the counter reaches <limit>...
		if (Scarcity_Samurai_Counter::limited($counter['counter_id'])) {
			switch ($_POST['ss-page-counter-expired']) {
				case 'do_nothing':
					$counter['action'] = array(
						'do_nothing' => true
					);

					break;

				case 'redirect_to_page':
					if ($_POST['ss-page-counter-expired-redirect-page-id'] === '') {
						$counter['action'] = array(
							'do_nothing' => true
						);
					} else {
						$counter['action'] = array(
							'redirect' => array(
								'page_id' => intval($_POST['ss-page-counter-expired-redirect-page-id'])
							)
						);
					}

					break;

				case 'redirect_to_url':
					$redirect_url = trim($_POST['ss-page-counter-expired-redirect-url']);

					if ($redirect_url === '') {
						$counter['action'] = array(
							'do_nothing' => true
						);
					} else {
						$counter['action'] = array(
							'redirect' => array(
								'url' => Scarcity_Samurai_Helper::add_http_to_url_if_required($redirect_url)
							)
						);
					}

					break;
			}
		}

		// -----------------
		// ---| Banners |---
		// -----------------

		foreach (array('header', 'footer') as $header_footer) {
			if ($_POST["ss-page-${header_footer}-banner-id"] === '') {
				continue;
			}

			$data = array(
				'enabled' => (isset($_POST["ss-page-${header_footer}-banner-enable"]) &&
				              ($_POST["ss-page-${header_footer}-banner-enable"] === 'on')),
				'banner_id' => intval($_POST["ss-page-${header_footer}-banner-id"])
			);

			switch ($_POST["ss-page-${header_footer}-banner-action"]) {
				case 'do_nothing':
					$data['action'] = array(
						'do_nothing' => true
					);

					break;

				case 'redirect_to_page':
					if ($_POST["ss-page-${header_footer}-banner-action-redirect-page-id"] === '') {
						$data['action'] = array(
							'do_nothing' => true
						);
					} else {
						$data['action'] = array(
							'redirect' => array(
								'page_id' => intval($_POST["ss-page-${header_footer}-banner-action-redirect-page-id"])
							)
						);
					}

					break;

				case 'redirect_to_url':
					$redirect_url = trim($_POST["ss-page-${header_footer}-banner-action-redirect-url"]);

					if ($redirect_url === '') {
						$data['action'] = array(
							'do_nothing' => true
						);
					} else {
						$data['action'] = array(
							'redirect' => array(
								'url' => Scarcity_Samurai_Helper::add_http_to_url_if_required($redirect_url)
							)
						);
					}

					break;
			}

			// Create or update the header/footer banner
			if ($header_footer === 'header') {
				$banner_attributes = Scarcity_Samurai_Page::header_banner_attributes($page_id);
				$position = 'fixed_top';
			} else {
				$banner_attributes = Scarcity_Samurai_Page::footer_banner_attributes($page_id);
				$position = 'fixed_bottom';
			}

			if ($banner_attributes === null) {
				$data['page_id'] = $page_id;
				$data['position'] = $position;

				Scarcity_Samurai_Model::get( 'Pages_Banners' )->insert($data);
			} else {
				Scarcity_Samurai_Model::get( 'Pages_Banners' )->update($data, $wpdb->prepare("page_id = %d AND position = %s", $page_id, $position));
			}
		}

		// ---------------
		// ---| Other |---
		// ---------------

		$data = $page['data'];
		$data['contains_opt_in_form'] =
			isset($_POST['ss-page-optin']) && ($_POST['ss-page-optin'] === 'on');

		$data['counter'] = $counter;

		$update_data = compact('campaign_id', 'type_id', 'available_from', 'available_until', 'data');

		if (!Scarcity_Samurai_Model::get( 'Page' )->update($update_data, $wpdb->prepare('id = %d', $page_id))) {
			Scarcity_Samurai_Helper::error('Page update failed!');
		}
	}

}
