<?php
/*
Plugin Name: Scarcity Samurai
Plugin URI:
Description: A simple marketing campaign creator purpose built to harness the power of scarcity to skyrocket your sales, profit, and income by two, three, even four times what you're currently making.
Version: 1.3.14
Author: Noble Samurai
Author URI: http://noblesamurai.com
*/

/* LEGAL COPYRIGHT NOTICE

Copyright (c) Noble Samurai Pty Ltd, 2008-2013. All Rights Reserved.

This software is proprietary to and embodies the confidential technology of Noble Samurai Pty Ltd.
Possession, use, dissemination or copying of this software and media is authorised only pursuant
to a valid written license from Noble Samurai Pty Ltd. Questions or requests regarding permission may
be sent by email to legal@noblesamurai.com or by post to PO Box 477, Blackburn Victoria 3130, Australia.

*/

require_once(ABSPATH . 'wp-admin/includes/plugin.php');

define('SCARCITY_SAMURAI_MIN_PHP_VERSION', '5.3.0');
define('SCARCITY_SAMURAI_MIN_WP_VERSION', '3.5');
global $wp_version;

$scarcity_samurai_php_is_outdated = version_compare(PHP_VERSION, SCARCITY_SAMURAI_MIN_PHP_VERSION, '<');
$scarcity_samurai_wp_is_outdated = version_compare($wp_version, SCARCITY_SAMURAI_MIN_WP_VERSION, '<');

if ($scarcity_samurai_php_is_outdated || $scarcity_samurai_wp_is_outdated) {
	if (is_admin()) {
		function scarcity_samurai_requirements_message() {
			global $wp_version, $scarcity_samurai_php_is_outdated, $scarcity_samurai_wp_is_outdated;

			$running = array();
			$need = array();
			$php_help = '';
			$wp_help = '';

			if ($scarcity_samurai_php_is_outdated) {
				$running[] = 'PHP ' . PHP_VERSION;
				$need[] = 'PHP ' . SCARCITY_SAMURAI_MIN_PHP_VERSION . ' or later';
				$php_help = "
					<p>
						For support updating your version of PHP, you can visit
						<a href='//noblesamurai.com/c/php-version' target='_blank'>our knowledgebase</a>.
					</p>
				";
			}

			if ($scarcity_samurai_wp_is_outdated) {
				$running[] = "WordPress $wp_version";
				$need[] = 'WordPress ' . SCARCITY_SAMURAI_MIN_WP_VERSION . ' or later';
				$wp_help = "
					<p>
						For support updating your version of WordPress, you can visit
						<a href='//noblesamurai.com/c/wordpress-version' target='_blank'>our knowledgebase</a>.
					</p>
				";
			}

			$running = join(' and ', $running);
			$need = join(' and ', $need);

			echo "
				<div class='error'>
					<p>
						Scarcity Samurai has detected that you are running $running.
					</p>
					<p>
						To activate Scarcity Samurai you need to be running $need.
					</p>
					$php_help
					$wp_help
				</div>
			";
		}

		add_action('admin_notices', 'scarcity_samurai_requirements_message');
	}

	deactivate_plugins(plugin_basename(__FILE__));
	unset($_GET['activate']); // Disable the 'Plugin activated' message

} else {
	// need to wrap in a try block otherwise it fails with an old version of
	// php before we get the above error message
	try {
		# values required for plugin update
		define( 'SS_PLUGIN_FILE', __FILE__ );
		define( 'SS_PLUGIN_UPDATE_PATH', 'http://plugins.noblesamurai.com/update.php' );

		global $wpdb, $charset_collate;

		if ( method_exists( $wpdb, 'get_charset_collate' ) ) {
			$scarcity_samurai_charset_collate = $wpdb->get_charset_collate();
		} else {
			require_once( ABSPATH . '/wp-admin/includes/schema.php' );
			$scarcity_samurai_charset_collate = $charset_collate;
		}

		require_once('class-scarcity-samurai.php');

		// See: http://wordpress.org/support/topic/register_activation_hook-does-not-work#post-1226615
		register_activation_hook(__FILE__, 'scarcity_samurai_on_activate');

		function scarcity_samurai_on_activate() {
			// The order is important because of foreign key constraints.
			Scarcity_Samurai_User::on_activate();
			Scarcity_Samurai_Token::on_activate();
			Scarcity_Samurai_Campaign::on_activate();
			Scarcity_Samurai_Page_Type::on_activate();
			Scarcity_Samurai_Page::on_activate();
			Scarcity_Samurai_Users_Subscriptions::on_activate();
			Scarcity_Samurai_Counter::on_activate();
			Scarcity_Samurai_Counters_Access::on_activate();
			Scarcity_Samurai_Settings::on_activate();
			Scarcity_Samurai_Banner::on_activate();
			Scarcity_Samurai_Banner_Element::on_activate();
			Scarcity_Samurai_Pages_Banners::on_activate();

			$first_time_activation = (get_site_option('scarcity_samurai_version') === false);

			// If the plugin is activated for the first time, seed the database.
			if ($first_time_activation) {
				Scarcity_Samurai::add_default_settings();
				Scarcity_Samurai::add_default_page_types();
				Scarcity_Samurai::add_default_banners();

				update_site_option('scarcity_samurai_db_version', Scarcity_Samurai::DATABASE_VERSION);
			}
		}

		$scarcity_samurai = new Scarcity_Samurai();
	} catch ( Exception $e ) {
	}
}
